/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * File:  "$Id: cav_api.c,v 1.3 2009/07/15 09:30:54 webmail Exp $"
 *
 * First Created on 2008/11/11 by jonas_liu@openfind.com.tw
 * Last Modified on "$Date: 2009/07/15 09:30:54 $" by  $Author: webmail $
 *
 * Copyright (c)2009 Openfind Information Technology INC.
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <netdb.h>

#include "define.h"          
#include "version.h"

#include "cav.h"
#include "cav_log.h"
#include "cav_log.c" /* cannot be gcc-linked */

#include "clamav.h"

M2K_VER_ID("$Id: cav_api.c,v 1.3 2009/07/15 09:30:54 webmail Exp $");

static struct cl_engine *sg_pstEngine = NULL;
static struct cl_stat sg_stDbStat;

char *cli_gentemp(char *);

#if 0
#ifdef __FREEBSD4
/* cross compiler compat */
int *__h_error(void) {
  return &h_errno;
}
#endif
#endif

static int
s_WriteDataToTmp(const char *pszData, int nDataSize, char *pszFile, int nFileSize) {
	char	szTempFile[512];
	char	*pcFile;
	int	nWriteSize;
	int	nFD;
	int	i;

	for(i=0; ; i++) {
		strncpy(szTempFile, CLAM_AV_TMP_FILE, sizeof(szTempFile)-1);
		pcFile = mktemp(szTempFile);
		if((nFD = open(pcFile, O_CREAT | O_EXCL | O_WRONLY, MAIL_FILE_PERMISSION)) < 0) {
			if(errno != EEXIST)
				return -1;

			/* to avoid fall in infinity loop */
			if(i > 128) {
				return -1;
			}
			continue;
		}
		break;
	}

	nWriteSize = write(nFD, pszData, nDataSize);
	close(nFD);

	if(nWriteSize != nDataSize) {
		unlink(pcFile);
		return -1;
	}
	else {
		strncpy(pszFile, pcFile, nFileSize-1);
		return 0;
	}
}

static unsigned int
s_GetScanOpt() {
	unsigned int	nOpt=0;
	
	/* refer to: http://www.clamav.org/doc/latest/clamdoc.pdf */
	/* CL_SCAN_STDOPT (defined in clamav.h):
			CL_SCAN_ARCHIVE
			CL_SCAN_MAIL
			CL_SCAN_OLE2
			CL_SCAN_PDF
			CL_SCAN_HTML
			CL_SCAN_PE
			CL_SCAN_ALGORITHMIC
			CL_SCAN_ELF
	*/
	
	nOpt |= CL_SCAN_STDOPT;
	//nOpt |= CL_SCAN_RAW;
	//nOpt |= CL_SCAN_ARCHIVE;
	//nOpt |= CL_SCAN_MAIL;
	//nOpt |= CL_SCAN_MAILURL;
	//nOpt |= CL_SCAN_OLE2;
	//nOpt |= CL_SCAN_BLOCKENCRYPTED;
	//nOpt |= CL_SCAN_PE;
	//nOpt |= CL_SCAN_ELF;
	//nOpt |= CL_SCAN_PDF;
	//nOpt |= CL_SCAN_BLOCKBROKEN;
	//nOpt |= CL_SCAN_HTML;
	//nOpt |= CL_SCAN_ALGORITHMIC;
	//nOpt |= CL_SCAN_PHISHING_BLOCKSSL;
	//nOpt |= CL_SCAN_PHISHING_BLOCKCLOAK;
	//nOpt |= CL SCAN_STRUCTURED;
	//nOpt |= CL SCAN_STRUCTURED_SSN_NORMAL;
	//nOpt |= CL SCAN_STRUCTURED_SSN_STRIPPED;
	//nOpt |= CL SCAN_PARTIAL_MESSAGE;
	//nOpt |= CL SCAN_HEURISTIC_PRECEDENCE;
	
	return nOpt;
}

static int
s_SetEngineOptions(struct cl_engine *pstEngine) {
	/* 
		availiable engine options:
		
		CL_ENGINE_MAX_SCANSIZE,	    // uint64_t 
		CL_ENGINE_MAX_FILESIZE,	    // uint64_t 
		CL_ENGINE_MAX_RECURSION,	 // uint32_t	
		CL_ENGINE_MAX_FILES,	    	 // uint32_t 
		CL_ENGINE_MIN_CC_COUNT,	    // uint32_t 
		CL_ENGINE_MIN_SSN_COUNT,	 // uint32_t 
		CL_ENGINE_PUA_CATEGORIES,	 // (char *) 
		CL_ENGINE_DB_OPTIONS,	    // uint32_t 
		CL_ENGINE_DB_VERSION,	    // uint32_t 
		CL_ENGINE_DB_TIME,		    // time_t 
		CL_ENGINE_AC_ONLY,		    // uint32_t 
		CL_ENGINE_AC_MINDEPTH,	    // uint32_t 
		CL_ENGINE_AC_MAXDEPTH,	    // uint32_t 
		CL_ENGINE_TMPDIR,		       // (char *) 
		CL_ENGINE_KEEPTMP		       // uint32_t 
		
		API: 
			int cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field field, long long num);
			long long cl_engine_get_num(const struct cl_engine *engine, enum cl_engine_field field, int *err);
			int cl_engine_set_str(struct cl_engine *engine, enum cl_engine_field field, const char *str);
			const char *cl_engine_get_str(const struct cl_engine *engine, enum cl_engine_field field, int *err);
	*/
	
	cl_engine_set_num(pstEngine, CL_ENGINE_MAX_FILES, 1000);
	cl_engine_set_num(pstEngine, CL_ENGINE_MAX_FILESIZE, 10 * 1048576);
	cl_engine_set_num(pstEngine, CL_ENGINE_MAX_RECURSION, 16);
	cl_engine_set_num(pstEngine, CL_ENGINE_MAX_SCANSIZE, 100 * 1048576);

	INFO("EngineOption: CL_ENGINE_MAX_SCANSIZE = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_MAX_SCANSIZE, NULL));
	INFO("EngineOption: CL_ENGINE_MAX_FILESIZE = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_MAX_FILESIZE, NULL));
	INFO("EngineOption: CL_ENGINE_MAX_RECURSION = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_MAX_RECURSION, NULL));
	INFO("EngineOption: CL_ENGINE_MAX_FILES = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_MAX_FILES, NULL));
	INFO("EngineOption: CL_ENGINE_MIN_CC_COUNT = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_MIN_CC_COUNT, NULL));
	INFO("EngineOption: CL_ENGINE_MIN_SSN_COUNT = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_MIN_SSN_COUNT, NULL));
	INFO("EngineOption: CL_ENGINE_PUA_CATEGORIES = %s", cl_engine_get_str(pstEngine, CL_ENGINE_PUA_CATEGORIES, NULL));
	INFO("EngineOption: CL_ENGINE_DB_OPTIONS = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_DB_OPTIONS, NULL));
	INFO("EngineOption: CL_ENGINE_DB_VERSION = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_DB_VERSION, NULL));
	INFO("EngineOption: CL_ENGINE_DB_TIME = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_DB_TIME, NULL));
	INFO("EngineOption: CL_ENGINE_AC_ONLY = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_AC_ONLY, NULL));
	INFO("EngineOption: CL_ENGINE_AC_MINDEPTH = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_AC_MINDEPTH, NULL));
	INFO("EngineOption: CL_ENGINE_AC_MAXDEPTH = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_AC_MAXDEPTH, NULL));
	INFO("EngineOption: CL_ENGINE_TMPDIR = %s", cl_engine_get_str(pstEngine, CL_ENGINE_TMPDIR, NULL));
	INFO("EngineOption: CL_ENGINE_KEEPTMP = %lld", cl_engine_get_num(pstEngine, CL_ENGINE_KEEPTMP, NULL));

	return 0;
}

/**************/
/* Public API */
/**************/

/* Return:
 *   0: Success
 *  -1: Error
**/
int 
CAV_ScanInit(int bForceReload) {
	unsigned int nSigs = 0;
	char	*pcTmp;
	int	nRet;

	INFO("trying to initialize the ClamAV engine, force_reload=%d...", bForceReload);

	/* check if already initizated */
	if(sg_pstEngine) {
		if(bForceReload) {
			INFO("handle is already initialized, close it...");
			CAV_Terminate();
		}
		else {
			INFO("engine is already initialized.");
			return 0;
		}
	}
	
	INFO("initializating ClamAV engine...");
	
	/* init ClamAV library */
	if(CL_SUCCESS != (nRet = cl_init(CL_INIT_DEFAULT))) {
		ERROR("cl_init(): error: %s (%d)", cl_strerror(nRet), nRet);
		return -1;
	}

	/* allocate new engine handler */		
	if(!(sg_pstEngine = cl_engine_new())) {
		ERROR("cl_engine_new() failed.");
		return -1;
	}
	
	/* set engine options */
	s_SetEngineOptions(sg_pstEngine);
		
	/* load all available databases from default directory */
	if(0 != (nRet = cl_load(CAV_DATABASE_DIR, sg_pstEngine, &nSigs, CL_DB_STDOPT))) {
		ERROR("cl_load(): error: %s (%d).", cl_strerror(nRet), nRet);
		return -1;
	}
	INFO("cl_load() done, %u signatures.", nSigs);

	/* build engine */
	if(0 != (nRet = cl_engine_compile(sg_pstEngine))) {
		ERROR("cl_engine_compile(): database initialization error: %s (%d)", cl_strerror(nRet), nRet);
		
		cl_engine_free(sg_pstEngine);
		sg_pstEngine = NULL;
		return -1;
	}
	
	/* update dbstat */
	memset(&sg_stDbStat, 0, sizeof(struct cl_stat));
	cl_statinidir(CAV_DATABASE_DIR, &sg_stDbStat);
	
	pcTmp = cli_gentemp(NULL);
	INFO("initializating ClamAV engine done, tmpdir=%s", pcTmp ? pcTmp : "Unknown");
	
	return 0;
}

/* return value:
 *  0: no virus found
 *  1: virus found
 * -1: error occurs
**/
int
CAV_Scan(const char *pszFileName, CAV_SCAN_RESULT *pstResult) {
	unsigned long int	nSize = 0;
	const char			*pcVirname;
	int	nRet, nCnt, nError;
	static int	bDoSrand=1;

	INFO("start to scan [%s], srand=%d...", pszFileName, bDoSrand);

	if(bDoSrand) {
		struct timeval tv;
		unsigned seed = 0;
		
		gettimeofday(&tv, (struct timezone *) 0);
		seed = tv.tv_usec + clock() + getpid();
		srand(seed);
		
		INFO("srand to seed %u.", seed);
		bDoSrand = 0;
	}
	
	if(!pszFileName || !pstResult) {
		ERROR("invalid parameters, name=%p, pstResult=%p.", pszFileName, pstResult);
		return -1;
	}
	
	memset(pstResult, 0, sizeof(pstResult));
	strncpy(pstResult->szSrcFile, pszFileName, sizeof(pstResult->szSrcFile)-1);
	pstResult->bInfected = 0;

	/* check database stat */
	if(cl_statchkdir(&sg_stDbStat)) {
		ERROR("cl_statchkdir(), database stat is changed.");
	}

#if 0
#error ClamAV 0.95 does not support Limits structure, use limits api instead.
	memset(&stLimits, 0, sizeof(struct cl_limits));
	stLimits.maxfiles = 1000; 				/* max files */
	stLimits.maxfilesize = 10 * 1048576;	/* maximum size of archived/compressed file */
	stLimits.maxreclevel = 16;					/* maximum recursion level for archives */
	stLimits.maxscansize = 100 * 1048576;	/*100MB*/
#endif

	/* scan file descriptor */
	nError = 0;
	nRet = cl_scanfile(pszFileName, &pcVirname, &nSize, sg_pstEngine, s_GetScanOpt());
	switch(nRet) {
		case CL_VIRUS:
			nCnt = pstResult->nVirusCnt;
			strncpy(pstResult->szVirusName[nCnt], pcVirname, sizeof(pstResult->szVirusName[nCnt])-1);
			snprintf(pstResult->szVirusPath[nCnt], sizeof(pstResult->szVirusPath[nCnt])-1, "%s", pszFileName);
			
			pstResult->nVirusCnt++;
			pstResult->bInfected = 1;

			INFO("virus detected on [%s]: (%s).", pszFileName, pcVirname);
			break;
			
		case CL_CLEAN:
			INFO("no virus detected on [%s].", pszFileName);
			break;

		default:
			ERROR("cl_scanfile() error: %s (%d)", cl_strerror(nRet), nRet);
			errno = nRet;
			nError = 1;
			break;
	}

	INFO("scan done, size=%u, infected=%d (%s), is_err=%d, last_errno=%d.", nSize, pstResult->bInfected, pstResult->szVirusName[0], nError, errno);
	return (nError ? -1 : (pstResult->bInfected ? 1 : 0));
}

/* return value:
 *  0: no virus found
 *  1: virus found
 * -1: error occurs
**/
int 
CAV_ScanBuf(const char *pszFileName, char *pszBuf, int nBufSize, CAV_SCAN_RESULT *pstResult) {
	char	szTmpFileName[512];
	int	nRet;

	INFO("start to scan buf, size=%d (%s)...", nBufSize, pszFileName);
	
	s_WriteDataToTmp(pszBuf, nBufSize, szTmpFileName, sizeof(szTmpFileName));
	INFO("save (%s) to tmpfile (%s)...", pszFileName, szTmpFileName);
	
	nRet = CAV_Scan(szTmpFileName, pstResult);
	
	INFO("remove tmpfile (%s)...", szTmpFileName);
	unlink(szTmpFileName);
	
	return nRet;
}

int 
CAV_Terminate() {
	INFO("trying to stop ClamAV engine...");

	/* uninit handle */
	if(sg_pstEngine) {
		INFO("trying to stop ClamAV engine, free engine handler.");
		cl_engine_free(sg_pstEngine);
		
		INFO("trying to stop ClamAV engine, clean engine pointer.");
		sg_pstEngine = NULL;
	
		INFO("trying to stop ClamAV engine, clean database stat.");	
		memset(&sg_stDbStat, 0, sizeof(struct cl_stat));

		INFO("trying to stop ClamAV engine, done.");
	}
	else {
		INFO("engine is not initialized.");
	}
	
	return 0;
}
