/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * File:  "$Id: cav_local.c,v 1.2 2009/07/15 09:31:18 webmail Exp $"
 *
 * First Created on 2008/11/11 by jonas_liu@openfind.com.tw
 * Last Modified on "$Date: 2009/07/15 09:31:18 $" by  $Author: webmail $
 *
 * Copyright (c)2009 Openfind Information Technology INC.
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <errno.h>

#include "cav.h"
#include "version.h"

M2K_VER_ID("$Id: cav_local.c,v 1.2 2009/07/15 09:31:18 webmail Exp $");

// return value:
// -2: cav library failure or permanent failure, errno indicator
// -1: temporary failure. errno indicator
//  0: no virus found
//  1: virus found but no action
//  2: virus found and partially repaired
//  3: virus found and fully repaired
// 
int
CAV_LocalScan(const char *pszFileName, char *pszMemBlock, int nMemSize, char *pszMsg, int nMsgSize, int nScanMode) {
	CAV_SCAN_RESULT	stResult;
	int					nRet;
	
	if(CAV_ScanInit(0) != 0)
		return -2;

	memset(&stResult, 0, sizeof(CAV_SCAN_RESULT));
	nRet = CAV_ScanBuf(pszFileName, pszMemBlock, nMemSize, &stResult);
	
	/* error occur */
	if(nRet == -1)
		return -2;
		
	/* no virus is found */
	if(nRet == 0)
		return 0;
		
	/* virus found, try to log all virus names */
	if(pszMsg && nMsgSize>0) {
		int nLen=0, i;
		
		for(i=0; i<stResult.nVirusCnt && i<MAX_LOG_VIRUS; i++) {
			if(strlen(stResult.szVirusName[i])+strlen(stResult.szVirusPath[i])+nLen+4 >= nMsgSize) {
				if(strlen(stResult.szVirusName[i])+nLen+5 < nMsgSize) {
					nLen += snprintf(pszMsg+nLen, nMsgSize-nLen-1, " \t%s\t%d\n", stResult.szVirusName[i], nRet);
				}
			}
			else {
				nLen += snprintf(pszMsg+nLen, nMsgSize-nLen-1, "%s\t%s\t%d\n", stResult.szVirusPath[i], stResult.szVirusName[i], nRet);
			}
		}
	}
	return 1;
}
