/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * File:  "$Id: cav_scan.c,v 1.2 2009/07/15 09:32:39 webmail Exp $"
 *
 * First Created on 2008/11/11 by jonas_liu@openfind.com.tw
 * Last Modified on "$Date: 2009/07/15 09:32:39 $" by  $Author: webmail $
 *
 * Copyright (c)2009 Openfind Information Technology INC.
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <errno.h>

#include "cav.h"
#include "version.h"

M2K_VER_ID("$Id: cav_scan.c,v 1.2 2009/07/15 09:32:39 webmail Exp $");


// initial string is:
// scan (\t)initial value(\0)

static int	sg_nScanSeq;

int
TAV_Init(const char *pszInitStr)
{
	char	szSeq[64];
	char	*pcPos;
	char	*pszServerList = NULL;
	
	pcPos = strchr(pszInitStr, '\t');
	if ( pcPos == NULL ){
		strncpy(szSeq, pszInitStr, sizeof(szSeq)-1);
	}
	else{
		int	nLen = pcPos - pszInitStr;

		if (nLen > (sizeof(szSeq) - 1))
			nLen = sizeof(szSeq) - 1;

		strncpy(szSeq, pszInitStr, nLen);
		szSeq[nLen] = '\0';
		pszServerList = pcPos + 1;
	}
	
	if ( strcasecmp(szSeq, "tcp") == 0 ){
		// TCP only
		sg_nScanSeq = 1;
		CAV_TcpInit(pszServerList);
	}
	else if ( strcasecmp(szSeq, "local") == 0 ){
		// Local only
		sg_nScanSeq = 2;
	}
	else if ( strcasecmp(szSeq, "tcp:local") == 0 ){
		// 1: TCP, 2: Local
		sg_nScanSeq = 3;
		CAV_TcpInit(pszServerList);
	}
	else if ( strcasecmp(szSeq, "local:tcp") == 0 ){
		// 1: Local, 2: TCP
		sg_nScanSeq = 4;
		CAV_TcpInit(pszServerList);
	}
	else{
		sg_nScanSeq = 2;
	}

	
	return 1;
}

int
TAV_Init2(const char *pszInitStr)
{
	char	szSeq[64];
	char	*pcPos;
	char	*pszServerList = NULL;
	
	pcPos = strchr(pszInitStr, '\t');
	if ( pcPos == NULL ){
		strncpy(szSeq, pszInitStr, sizeof(szSeq)-1);
	}
	else{
		int	nLen = pcPos - pszInitStr;

		if (nLen > (sizeof(szSeq) - 1))
			nLen = sizeof(szSeq) - 1;

		strncpy(szSeq, pszInitStr, nLen);
		szSeq[nLen] = '\0';
		pszServerList = pcPos + 1;
	}
	
	if ( strcasecmp(szSeq, "tcp") == 0 ){
		// TCP only
		sg_nScanSeq = 1;
		CAV_TcpInit(pszServerList);
	}
	else if ( strcasecmp(szSeq, "local") == 0 ){
		// Local only
		sg_nScanSeq = 2;
		CAV_ScanInit(1);
	}
	else if ( strcasecmp(szSeq, "tcp:local") == 0 ){
		// 1: TCP, 2: Local
		sg_nScanSeq = 3;
		CAV_TcpInit(pszServerList);
	}
	else if ( strcasecmp(szSeq, "local:tcp") == 0 ){
		// 1: Local, 2: TCP
		sg_nScanSeq = 4;
		CAV_ScanInit(1);
		CAV_TcpInit(pszServerList);
	}
	else{
		sg_nScanSeq = 2;
		CAV_ScanInit(1);
	}

	
	return 1;
}

int
TAV_Close(void)
{
	/* CAV_TcpScan never close tcp connection, if virus scan result
	 * of sph_srv equal to "201"
	 * Close tcp connection to avoid file descriptor leaking, 
	 */
	CAV_TcpDisconnect();
	return CAV_Terminate();
}

// return value:
// -2:
// -1: encounter error
//  0: no virus found
//  1: virus found but no action
//  2: virus found and partially repaired
//  3: virus found and fully repaired
int
TAV_Scan(const char *psMemBlock, const unsigned long unSize, const char *pszFileName,
	const char *psPattern, const unsigned long unPsize,
	char *pszMsg, const int nMsgSize, int nScanMode, 
	char **ppsRepairedFile, unsigned int *punNewSize)
{
	int	nRet;
	
	// 1. first try TCP scan 
	switch( sg_nScanSeq ){
	case 4:
		// local first, then TCP
		nRet = CAV_LocalScan(pszFileName, (char *)psMemBlock, unSize, pszMsg, nMsgSize, nScanMode);
		/*dont continue scan when success, and Sophos Error*/
		/*The same Sophos Erorr will occur in continue scanning*/
		if ( nRet >= 0 || nRet == -2 ){
			break;
		}
	case 1:	
		nRet = CAV_TcpScan(pszFileName, (char *)psMemBlock, unSize, pszMsg, nMsgSize, nScanMode);
		break;
	case 3:
		// TCP first, then local
		nRet = CAV_TcpScan(pszFileName, (char *)psMemBlock, unSize, pszMsg, nMsgSize, nScanMode);
		/*dont continue scan when success, and Sophos Error*/
		/*The same Sophos Erorr will occur in continue scanning*/
		if ( nRet >= 0 || nRet == -2 ){
			break;
		}
	case 2:
		nRet = CAV_LocalScan(pszFileName, (char *)psMemBlock, unSize, pszMsg, nMsgSize, nScanMode);
		break;
	}

	if ( nRet > 0 && pszMsg[0] == '\t' ){ /*if filename is empty*/
		char m_buf[512];
		strncpy(m_buf, pszMsg, sizeof(m_buf)-1);
		snprintf(pszMsg, nMsgSize - 1, "%s%s", pszFileName, m_buf);
	}
	// if the file is changed	
	// reopen the file and allocate the memory buffer
	if ( ppsRepairedFile ){
		*ppsRepairedFile = NULL;
	}
	if ( punNewSize ){
		*punNewSize = 0;
	}
	
	return nRet;
}

#ifdef	__MAIN_PROG

#include <libgen.h>

int
main(int argc, char *argv[])
{
	int	i;
	int	fd;
	int	nRet;
	char	*psMemBlock;
	char	*psClean;
	int	nSize;
	int	nCleanSize;
	char	szMsg[512];
	struct stat	sb;	
	
	if ( argc < 2 ){
		printf("usage: %s file-name [file-name]...\n", argv[0]);
		return 0;
	}
		
	TAV_Init("2\t127.0.0.1:5900");
	
	for(i=2; i<=argc; i++){
		psClean = NULL;
		if ( stat(argv[i-1], &sb) != 0 ){
			printf("file not exist\n");
			return -1;
		}
		nSize = sb.st_size;
		psMemBlock = malloc(nSize+2);
		if ( psMemBlock == NULL ){
			printf("error to allocate memory\n");
			return -1;
		}
		fd = open(argv[i-1], O_RDONLY);
		if ( fd < 0 ){
			printf("open file %s failed\n", argv[i]);
			continue;
		}
		read(fd, psMemBlock, nSize);
		close(fd);
		
		//
		nRet = TAV_Scan(psMemBlock, nSize, basename(argv[i]), NULL, 0,  szMsg, 512, 0, &psClean, &nCleanSize);
		free(psMemBlock);
		if ( nRet < 0 ){
			printf("scan failed\n");
		}
		else if ( nRet == 0 ){
			printf("file is clean");
		}
		else{
			printf("return code: %d\n", nRet);
			printf("%s", szMsg);
		}
	}
	
	return 0;
}

#endif

