/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * File:  "$Id: cav_tcp.c,v 1.3 2009/07/15 10:16:54 webmail Exp $"
 *
 * First Created on 2008/11/11 by jonas_liu@openfind.com.tw
 * Last Modified on "$Date: 2009/07/15 10:16:54 $" by  $Author: webmail $
 *
 * Copyright (c)2009 Openfind Information Technology INC.
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>

#include "tcp.h"  
#include "bfd.h"   
#include "toutio.h"

#include "cav.h"
#include "version.h"

M2K_VER_ID("$Id: cav_tcp.c,v 1.3 2009/07/15 10:16:54 webmail Exp $");


#define	CAV_AV_MAX_SERVER	4
#define	CAV_AV_MAX_SERVER_BUFSIZE	256
#define	CAV_AV_SERVER_NAME	"127.0.0.1"
#define	CAV_AV_SERVER_PORT	5900
#define	CAV_AV_TIMEOUT		10
#define	CAV_AV_NET_CHK(a, b)	if ( a <= 0 ){ CloseServer(b); return -1;}

static char	sg_szServerName[CAV_AV_MAX_SERVER][CAV_AV_MAX_SERVER_BUFSIZE];
static int	sg_nServerPort[CAV_AV_MAX_SERVER];
static int	sg_nServerCount;
static int	sg_nSocket = -1;
static int	sg_nErr = 0;
static struct bfd	sg_stBfd;

int
CAV_TcpInit(const char *pszServerList)
{
	char	szBuf[CAV_AV_MAX_SERVER_BUFSIZE * 2];
	char	*pcPos;
	int	i;
	
	if ( pszServerList == NULL || pszServerList[0] == '\0' ){
		strncpy(sg_szServerName[0], CAV_AV_SERVER_NAME, CAV_AV_MAX_SERVER_BUFSIZE-1);
		sg_nServerPort[0] = CAV_AV_SERVER_PORT;
		sg_nServerCount = 1;
		
		return 1;
	}
	
	sg_nErr = 0;
	sg_nServerCount = 0;
	while( pszServerList ){
		pcPos = strchr(pszServerList, ',');
		if ( pcPos == NULL ){
			strncpy(szBuf, pszServerList, sizeof(szBuf)-1);
			pszServerList = NULL;
		}
		else{
			int nMaxLen;
			i = pcPos - pszServerList;

			nMaxLen = (i > (sizeof(szBuf)-1)) ? (sizeof(szBuf)-1):i;

			strncpy(szBuf, pszServerList, nMaxLen);
			szBuf[i] = '\0';
			pszServerList = pcPos + 1;
		}
				
		pcPos = strchr(szBuf, ':');
		if ( pcPos != NULL ){
			*pcPos = '\0';
			sg_nServerPort[sg_nServerCount] = atoi(pcPos+1);
		}
		else{
			sg_nServerPort[sg_nServerCount] = CAV_AV_SERVER_PORT;
		}
		strncpy(sg_szServerName[sg_nServerCount], szBuf, CAV_AV_MAX_SERVER_BUFSIZE-1);
		sg_nServerCount++;
	}
	
	return 1;
}

static int
set_signal(int signo, void (*func)(int))
{
	struct sigaction act;

	act.sa_handler = func;
	sigemptyset(&act.sa_mask);
	act.sa_flags = SA_RESTART;
	
	return sigaction(signo, &act, NULL);
}

static void
DoNothing(int i)
{
	return;
}

static int
CloseServer()
{
	if (sg_nSocket >= 0) {
		twriten(sg_nSocket, "Quit\r\n", 6, CAV_AV_TIMEOUT);
		close(sg_nSocket);
		sg_nSocket = -1;
	}
	return 0;
}

// int ConnectServer()
// 1. try to connect Symantec Anti-virus server via TCP/IP
// 2. get the greeting message if connection is built.
// 
// return value
//  0: success
// -1: failed while connecting server
static int
ConnectServer()
{
	int	nRet;
	char	szRecvBuf[1024];
	int	i;

	if ( sg_nSocket > 0 ){
		return 0;
	}
	
	set_signal(SIGALRM, DoNothing);	
	// try to connect server
	for(i=0; i<sg_nServerCount; i++){
		alarm(CAV_AV_TIMEOUT);
		sg_nSocket = tcp_conn(sg_szServerName[i], sg_nServerPort[i]);
		alarm(0);
		if ( sg_nSocket >= 0 ){
			// swap the buffer location, use the available server first
			if ( i > 0 ){
				strncpy(szRecvBuf, sg_szServerName[0], sizeof(szRecvBuf)-1);
				nRet = sg_nServerPort[0];
				
				strncpy(sg_szServerName[0], sg_szServerName[i], CAV_AV_MAX_SERVER_BUFSIZE-1);
				sg_nServerPort[0] = sg_nServerPort[i];
				
				strncpy(sg_szServerName[i], szRecvBuf, CAV_AV_MAX_SERVER_BUFSIZE-1);
				sg_nServerPort[i] = nRet;
			}
			break;
		}
	}
	if ( sg_nSocket < 0 ){
		return -1;
	}
	
	// Get the CAV server greeting
	bfd_init(&sg_stBfd, sg_nSocket);
	nRet = bfd_readrec(&sg_stBfd, szRecvBuf, 1023, '\n', 10);
	CAV_AV_NET_CHK(nRet, sg_nSocket);
	
	return 0;
}

int CAV_TcpDisconnect(void)
{
	return CloseServer();
}

// CAV_TcpScan()
// Call this function to scan file with ClamAV anti-virus technology.
// This function uses native protocol via TCP/IP to make a scanning requst.
// Connection is terminated after a single scan request is done.
// 
// Scaning procedures:
//  - Connecting to server and  hand-shaking
//  - Send request and target to be scanned to server
//  - Get the scan result
//  - Get the infection information if virus contained
//  - Get the reply code to see if output file available
//  - Receive the output file (object is repaired or some inside components are deleted)
// 
// return value
//  0: no virus found, object is clean
//  1: one or more virus found, no action is applied and object is not clean
//  2: one or more virus found, object is not clean due to partial component are not repairable.
//  3: one or more virus found, 
// -1: encounter error
// -2: 
int CAV_TcpScan(const char *pszFileName, char *pszMemBlock, int nMemSize, char *
pszMsg, int nMsgSize, int nScanMode)
{
	char	szSendBuf[1024];
	char	szRecvBuf[1024];
	char	szComponent[256];
	char	szInfectionName[256];
	char    szTempFile[256];
	char	*pszPos;
	int	i;
	int	nScanStatus;
	int	nInfectionCount;
	int	nLen;
	int	nResult = 0;
	char	c;
	int	nFd;

	if ( sg_nErr != 0 || pszMemBlock == NULL || nMemSize < 0 ){
		return -1;
	}
	
	/* no content, no virus */
	if(nMemSize == 0)
		return 0;
	
	if ( ConnectServer() != 0 ){
		sg_nErr = 1;
		return -1;
	}	
	
	/*if ( pszMsg ) pszMsg[0] = '\0';*/

	if ( (pszPos = strrchr(pszFileName, '.')) && strlen(pszPos) < 5 )
		snprintf(szTempFile, sizeof(szTempFile)-1, "%s/%d%s", CAV_TEMP_PATH, getpid(), pszPos);
	else
		snprintf(szTempFile, sizeof(szTempFile)-1, "%s/%d.xxx", CAV_TEMP_PATH, getpid());
	if ( (nFd = open(szTempFile, O_WRONLY|O_CREAT, 0600)) == -1 )
		return -1;
	write(nFd, pszMemBlock, nMemSize);
	close(nFd);
	
	// Send virus scan command and memory block
	if ( nScanMode == 2 ){
		snprintf(szSendBuf, sizeof(szSendBuf)-1, "SCANREPAIRDELETE %s\r\n", szTempFile);
	} else if ( nScanMode == 1 ){
		snprintf(szSendBuf, sizeof(szSendBuf)-1, "SCANREPAIR %s\r\n", szTempFile);
	} else{
		snprintf(szSendBuf, sizeof(szSendBuf)-1, "SCAN %s\r\n", szTempFile);
	}

	i = twriten(sg_nSocket, szSendBuf, strlen(szSendBuf), CAV_AV_TIMEOUT);
	if ( i <= 0 ){
		sg_nSocket = -1;
		if ( ConnectServer() != 0 ){
			sg_nErr = 1;
			unlink(szTempFile);
			return -1;
		}
		i = twriten(sg_nSocket, szSendBuf, strlen(szSendBuf), CAV_AV_TIMEOUT);
		CAV_AV_NET_CHK(i, sg_nSocket);
	}
	

	// Get the scan result code:
	// 201: file is clean.
	// 202: file is infected
	i = bfd_readrec(&sg_stBfd, szRecvBuf, 1023, '\n', CAV_AV_TIMEOUT);
	CAV_AV_NET_CHK(i, sg_nSocket);

	unlink(szTempFile);
	
	if ( strncmp(szRecvBuf, "201", 3) == 0 ){
		nScanStatus = 0;
		return 0;
	}
	else if ( strncmp(szRecvBuf, "202", 3) == 0 ){
		nScanStatus = 1;
	}
	else if ( strncmp(szRecvBuf, "203", 3) == 0 ){
		nScanStatus = 2;
	}
	else{
		CloseServer();
		errno = atoi(szRecvBuf);
		return -2;
	}
	
	switch(nScanStatus){
	case 0:
		break;
	case 1:
	case 2:
	case 3:
		nResult = 1;
		nLen = 0;
		nInfectionCount = 0;
		// Get the virus list
		while( 1 ){
			i = bfd_readrec(&sg_stBfd, szRecvBuf, 1023, '\n', CAV_AV_TIMEOUT);
			CAV_AV_NET_CHK(i, sg_nSocket);
			pszPos = strchr(szRecvBuf, '\r');
			if ( pszPos )	*pszPos = '\0';
			
			if ( strncmp(szRecvBuf, "204 ", 4) == 0 || strncmp(szRecvBuf, "205 ", 4) == 0 ){
				break;
			}
			
			// bit 1: infection status
			// 0: still infected
			// 1: infection fixed
			// 2: infection deleted
			switch ( (c=szRecvBuf[0]) ){
			case '0':
				break;
			case '1':
				break;
			case '2':
				break;
			default:
				break;
			}
			i = bfd_readrec(&sg_stBfd, szRecvBuf, 1023, '\n', CAV_AV_TIMEOUT);
			CAV_AV_NET_CHK(i, sg_nSocket);
			pszPos = strchr(szRecvBuf, '\r');
			if ( pszPos )	*pszPos = '\0';
			strncpy(szComponent, szRecvBuf, sizeof(szComponent)-1);
			
			i = bfd_readrec(&sg_stBfd, szRecvBuf, 1023, '\n', CAV_AV_TIMEOUT);
			CAV_AV_NET_CHK(i, sg_nSocket);
			pszPos = strchr(szRecvBuf, '\r');
			if ( pszPos )	*pszPos = '\0';
			strncpy(szInfectionName, szRecvBuf, sizeof(szInfectionName)-1);
			
			if ( pszMsg != NULL ){
				i = strlen(szComponent) + strlen(szInfectionName);
				if ( (nLen+i+4) < nMsgSize ){
					nLen += snprintf(pszMsg+nLen, nMsgSize-nLen-1, "%s\t%s\t%c\n", szComponent, szInfectionName, c);
				}
			}
			nInfectionCount++;
		}
		break;
	}
	
	CloseServer();
	
	return nResult;
}
