#!/usr/bin/perl
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#
# File:  "$Id: cav_update.pl,v 1.2 2009/07/15 09:35:26 webmail Exp $"
#
# First Created on 2008/11/11 by jonas_liu@openfind.com.tw
# Last Modified on "$Date: 2009/07/15 09:35:26 $" by  $Author: webmail $
#
# Copyright (c)2009 Openfind Information Technology INC.
# All rights reserved.
#

use strict;
use Getopt::Std;
use File::Basename;
use lib dirname($0);
use Openfind;

my $SYS_BASEDIR = OP_GetBaseDir();
my $MAILED_PID_FILE = OP_GetMailerdPidFile();
my $CAV_BASEDIR        = "$SYS_BASEDIR/ClamAV";
my $CAV_FRESHCLAM_BIN  = "$CAV_BASEDIR/bin/freshclam";
my $CAV_FRESHCLAM_CONF = "$CAV_BASEDIR/etc/freshclam.conf";
my $CAV_LOGFILE        = "$CAV_BASEDIR/log/cav_update.log";
my $CAVSRV_PID_FILE    = "$CAV_BASEDIR/log/cav_srv.pid";

my $VERBOSE_CONSOLE = 0;
my $VERBOSE_LOGFILE = 1;

sub MESG {
	my $szMesg = shift;
	my ($sec, $min, $hour, $day, $mon, $year) = localtime(time);
	my $szTimeStamp = sprintf("[%04d/%02d/%02d %02d:%02d:%02d]", $year+1900, $mon+1, $day, $hour, $min, $sec);
	
	# log to console
	print "$szTimeStamp $szMesg\n" if($VERBOSE_CONSOLE);
	
	# log to file
	OP_CatToFile($CAV_LOGFILE, "$szTimeStamp $szMesg") if($VERBOSE_LOGFILE);
}

sub RestartDaemons {
	my $nPID;
	
	foreach my $szPIDFile (($MAILED_PID_FILE, $CAVSRV_PID_FILE)) {
		MESG("restarting daemon ($szPIDFile)...");
		
		if(open(PIDIN, $szPIDFile)) {
			$nPID = <PIDIN>;
			$nPID += 0;
			close(PIDIN);
		
			if($nPID > 0 ) {
				kill(1, $nPID);
				MESG("sending HUP signal to daemon ($nPID)...");
			}
			else {
				MESG("pid of the daemon is invalid ($nPID).");
			}
		}
		else {
			MESG("cannot open pid file '$szPIDFile'.");
		}
	}
}

sub main {
	my %hOpt;
	getopts('v', \%hOpt);
	
	$VERBOSE_CONSOLE=1 if(defined($hOpt{'v'}));

	if(! -x $CAV_FRESHCLAM_BIN) {
		MESG("$CAV_FRESHCLAM_BIN is not executable.");
		return -1;
	}
		
	my $szRet = `$CAV_FRESHCLAM_BIN --quiet --on-update-execute='echo CAV_UPDATED' --config-file='$CAV_FRESHCLAM_CONF' --stdout`;
	
	if($szRet =~ m/CAV_UPDATED/g) {
		MESG("database is updated, try to restart daemons.");
		RestartDaemons();
		return 1;
	}
	else {	
		MESG("database is update-to-date.");
		return 0;
	}
}

exit(main());
