/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * File:  "$Id: bfd.c,v 1.1 2009/07/15 11:05:16 webmail Exp $"
 *
 * First Created on 2008/11/11 by jonas_liu@openfind.com.tw
 * Last Modified on "$Date: 2009/07/15 11:05:16 $" by  $Author: webmail $
 *
 * Copyright (c)2009 Openfind Information Technology INC.
 * All rights reserved.
 */

#include <sys/types.h>
#include <string.h>
#include "bfd.h"
#include "toutio.h"
#include "version.h"

M2K_VER_ID("$Id: bfd.c,v 1.1 2009/07/15 11:05:16 webmail Exp $");


int bfd_init(struct bfd *bd, int fd)
{
   bd->fd = fd;
   bd->widx = 0;
   bd->ridx = 0;
   return 1;
}

int bfd_write(struct bfd *bd, char *buf, size_t sz, int t, int flush)
{
   char *ptr;
   int  szleft;
   int  szwrite;

   if ( sz == 0 ) return 0;
   szleft = sz;
   ptr = buf;

   while ( szleft + bd->widx >= BFD_WBUF_SIZE ){
      szwrite = BFD_WBUF_SIZE - bd->widx;
      memcpy(bd->wbuf+bd->widx, ptr, szwrite);
      if ( twriten(bd->fd, bd->wbuf, BFD_WBUF_SIZE, t) <= 0 )
	 return -1;
      szleft -= szwrite;
      ptr += szwrite;
      bd->widx = 0;
   }
   memcpy(bd->wbuf+bd->widx, ptr, szleft);
   bd->widx += szleft;
   if ( flush ){
      if ( twriten(bd->fd, bd->wbuf, bd->widx, t) <= 0 )
	 return -1;
      else
	 bd->widx = 0;
   }
   return sz;
}

int bfd_flush(struct bfd *bd, int t)
{
   if ( bd->widx == 0 ) return 0;
   if ( twriten(bd->fd, bd->wbuf, bd->widx, t) <= 0 )
      return -1;
   bd->widx = 0;
   return bd->widx;
}


int bfd_read(struct bfd *bd, char *buf, size_t sz, int t) 
{
   char *ptr;
   int  szleft;
   int  nread;

   ptr = buf;
   szleft = sz;

   while ( szleft > bd->ridx ){
      memcpy(ptr, bd->rbuf, bd->ridx);
      szleft -= bd->ridx;
      ptr += bd->ridx;
      bd->ridx = 0;
      nread = tread(bd->fd, bd->rbuf, BFD_RBUF_SIZE, t);
      if ( nread == -1 ) return -1;
      if ( nread == 0 ) return (sz - szleft);
      bd->ridx = nread;
   }
   memcpy(ptr, bd->rbuf, szleft);
   bd->ridx -= szleft;
   memmove(bd->rbuf, &bd->rbuf[szleft], bd->ridx);
   return sz;
}

/*
  int bfd_readrec() -- read records seperated by character 'sep', and stores
                       into 'bsz' sized buffer 'buf'.
  struct bfd *bd : the buffered file descriptor.
  char *buf : the buffer to put extracted record.
  size_t bsz: the size of buffer.
  char sep  : the record seperator.
  int t     : the timeout interval.
  return 1 on success, 0 on end of record, -1 on error.
 */

int bfd_readrec(struct bfd *bd, char *buf, size_t bsz, char sep, int t)
{
   int  nread;
   char *rec_end;
   int  freesz;
   int  usedsz;

   freesz = bsz;
   usedsz = 0;

   while ( !(rec_end = memchr(bd->rbuf, sep, bd->ridx)) ){
      if ( bd->ridx >= freesz ){ /*exceeds buffer size before record end*/
         memcpy(buf+usedsz, bd->rbuf, freesz);
	 usedsz += freesz; bd->ridx -= freesz;
	 memcpy(bd->rbuf, bd->rbuf + freesz, bd->ridx);
	 buf[usedsz] = '\0';
	 return 1;
      }
      memcpy(buf+usedsz, bd->rbuf, bd->ridx);
      usedsz += bd->ridx; freesz -= bd->ridx;
      bd->ridx = 0;
      nread = tread(bd->fd, bd->rbuf, BFD_RBUF_SIZE, t);
      if ( nread == -1 ) return -1;
      if ( nread == 0 ) {
	 if ( usedsz == 0 ) return 0;
	 buf[usedsz] = '\0';
	 return 1;
      }
      bd->ridx = nread;
   }
   nread = rec_end - bd->rbuf;
   memcpy(buf+usedsz, bd->rbuf, nread); /*does not copy sep*/
   buf[nread+usedsz] = '\0';
   bd->ridx -= (nread + 1);/*pass the seperator character*/
   memmove(bd->rbuf, rec_end+1, bd->ridx);
   return 1;
}
