/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * File:  "$Id: toutio.c,v 1.1 2009/07/15 11:05:16 webmail Exp $"
 *
 * First Created on 2008/11/11 by jonas_liu@openfind.com.tw
 * Last Modified on "$Date: 2009/07/15 11:05:16 $" by  $Author: webmail $
 *
 * Copyright (c)2009 Openfind Information Technology INC.
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <sys/time.h>
#include <string.h>
#include <errno.h>
#include "toutio.h"
#include "version.h"

M2K_VER_ID("$Id: toutio.c,v 1.1 2009/07/15 11:05:16 webmail Exp $");




/*
   ssize_t tread() -- same as read(2), but there's timeout on read. 
   return bytes read on success, -1 on fail or timeout. 
 */
ssize_t tread(int fd, void *buf, size_t len, int secs)
{
   fd_set rset;
   struct timeval tval;
   size_t nread;

   FD_ZERO(&rset); 
   FD_SET(fd, &rset);
   tval.tv_sec = secs;
   tval.tv_usec = 0;
   errno=ETIMEDOUT; 
   if ( select(fd+1, &rset, 0, 0, &tval) <= 0 )
      return -1; /*timeout*/
   if ( FD_ISSET(fd, &rset) )
      return nread = read(fd, buf, len);
   return -1;
}


/*
   ssize_t treadn() -- same as readn, but there's timeout on read. 
   return bytes read on success, -1 on fail or timeout. 
 */
ssize_t treadn(int fd, void *buf, size_t len, int secs)
{
   size_t nread, szleft;
   int n;

   szleft = len;
   nread = 0;
   errno=ETIMEDOUT ;
   while ( szleft > 0 ) {
      n=tread(fd, buf+nread, szleft, secs);
      if ( n <= 0 ) return -1;
      szleft -= n;
      nread += n;
   }
   return nread;
}
/*
   ssize_t twrite() -- same as write(2), but there's timeout on write. 
   return bytes read on success, -1 on fail or timeout. 
 */

ssize_t twrite(int fd, void *buf, size_t len, int secs)
{
   fd_set wset;
   struct timeval tval;
   size_t nwrite;

   FD_ZERO(&wset); 
   FD_SET(fd, &wset); 
   tval.tv_sec = secs;
   tval.tv_usec = 0;
   errno=ETIMEDOUT;
   if ( select(fd+1, 0, &wset, 0, &tval) <= 0 )
      return -1;/*timeout*/

   if ( FD_ISSET(fd, &wset) )
      return nwrite = write(fd, buf, len);
   return -1;
}


/*
   ssize_t twriten() -- same as writen, but there's timeout on write. 
   return bytes read on success, -1 on fail or timeout. 
 */
ssize_t twriten(int fd, void *buf, size_t len, int secs)
{
   size_t nwrite, szleft;
   int n;

   szleft = len;
   nwrite = 0;
   errno=ETIMEDOUT ;
   while ( szleft > 0 ) {
      n=twrite(fd, buf+nwrite, szleft, secs);
      if ( n <= 0 ) return -1;
      szleft -= n;
      nwrite += n;
   }
   return nwrite;
}
